# -*- coding: utf-8 -*-
"""
Created on Thu Sep 10 11:58:46 2015

In this example an actually measured signal is deconvolved and the result compared to a reference measurement signal.

.. moduleauthor:: Sascha Eichstaedt (sascha.eichstaedt@ptb.de)
"""

from matplotlib.pyplot import *
from GUM2DFT import GUM_DFT, GUM_iDFT, GUM_multiply

colors = [[0.1, 0.6, 0.5], [0.9, 0.2, 0.5], [0.9, 0.5, 0.1]]

# low-pass filter
def lowpass(f, fcut = 100e6):
	return 1 / (1 + 1j * f / fcut) ** 2


# %% measured hydrophone output signal
meas = np.loadtxt("data/measured_signal.dat")
time = meas[:,0]
y = meas[:, 1]
Ts = 2e-9
# assumed noise std
noise_std = 1e-3
Uy = np.ones(len(y))*noise_std ** 2

# %% filter for multiplication
Nf = 4096
f = np.fft.rfftfreq(Nf, Ts)
HL = lowpass(f)
HL = np.r_[np.real(HL), np.imag(HL)]

print("Propagating uncertainty associated with measurement through DFT")
Y, UY = GUM_DFT(y, Uy, N = Nf)
print('Propagating uncertainty through the multiplication in the DFT domain')
YL, UYL = GUM_multiply(Y, UY, HL)
print('Propagating uncertainty through inverse DFT')
yL, UyL = GUM_iDFT(YL, UYL, Nx = len(y))

uy  = np.sqrt(Uy)
uyL = np.sqrt(np.diag(UyL))

##%% plotting results

figure(1); clf()
plot(time * 1e6, y, label="original signal")
plot(time * 1e6, yL, label="filtered signal")
legend(loc="best")

figure(2); clf()
plot(np.sqrt(np.diag(UY[:Nf//2,:Nf//2])), "+", label="original signal", alpha=0.5)
plot(np.sqrt(np.diag(UYL.toarray()[:Nf//2,:Nf//2])), "o", label="filtered signal", alpha=0.2)
legend(loc="best")

figure(3);clf()
plot(time*1e6, uy, "+", label="original signal", alpha=0.5)
plot(time*1e6, uyL,"o", label="filtered signal", alpha=0.2)
legend(loc="best")
ylim(0,2*noise_std)

